@@ -3,18 +3,28 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, Type
+from typing import Optional, Type, Any
+from abc import ABC, abstractmethod
 import sys
 
-import sqlalchemy as sql
-from sqlalchemy.engine.base import Engine
+from mautrix.types import RoomID, UserID
+from mautrix.appservice import AppService, ASStateStore
 
-from mautrix.appservice import AppService
-from ..util.db import Base
 from ..util.program import Program
-from .db import SQLStateStore
 from .config import BaseBridgeConfig
 from .matrix import BaseMatrixHandler
+from .portal import BasePortal
+from .user import BaseUser
+from .puppet import BasePuppet
+
+try:
+    from .state_store.sqlalchemy import SQLBridgeStateStore
+    from ..util.db import Base
+
+    import sqlalchemy as sql
+    from sqlalchemy.engine.base import Engine
+except ImportError:
+    Base = SQLBridgeStateStore = sql = Engine = None
 
 try:
     import uvloop
@@ -22,11 +32,11 @@ except ImportError:
     uvloop = None
 
 
-class Bridge(Program):
-    db: Engine
+class Bridge(Program, ABC):
+    db: 'Engine'
     az: AppService
-    state_store_class: Type[SQLStateStore] = SQLStateStore
-    state_store: SQLStateStore
+    state_store_class: Type[ASStateStore] = SQLBridgeStateStore
+    state_store: ASStateStore
     config_class: Type[BaseBridgeConfig]
     config: BaseBridgeConfig
     matrix_class: Type[BaseMatrixHandler]
@@ -40,7 +50,7 @@ class Bridge(Program):
                  real_user_content_key: Optional[str] = None,
                  config_class: Type[BaseBridgeConfig] = None,
                  matrix_class: Type[BaseMatrixHandler] = None,
-                 state_store_class: Type[SQLStateStore] = None) -> None:
+                 state_store_class: Type[ASStateStore] = None) -> None:
         super().__init__(module, name, description, command, version, config_class)
         if real_user_content_key:
             self.real_user_content_key = real_user_content_key
@@ -66,8 +76,8 @@ class Bridge(Program):
 
     def prepare(self) -> None:
         super().prepare()
-        self.prepare_appservice()
         self.prepare_db()
+        self.prepare_appservice()
         self.prepare_bridge()
 
     def prepare_config(self) -> None:
@@ -82,13 +92,22 @@ class Bridge(Program):
         self.config.save()
         print(f"Registration generated and saved to {self.config.registration_path}")
 
+    def make_state_store(self) -> None:
+        if self.state_store_class is None:
+            raise RuntimeError("state_store_class is not set")
+        elif SQLBridgeStateStore and issubclass(self.state_store_class, SQLBridgeStateStore):
+            self.state_store = self.state_store_class(self.get_puppet, self.get_double_puppet)
+        else:
+            self.state_store = self.state_store_class()
+
     def prepare_appservice(self) -> None:
-        self.state_store = self.state_store_class()
+        self.make_state_store()
         mb = 1024 ** 2
         self.az = AppService(server=self.config["homeserver.address"],
                              domain=self.config["homeserver.domain"],
                              verify_ssl=self.config["homeserver.verify_ssl"],
 
+                             id=self.config["appservice.id"],
                              as_token=self.config["appservice.as_token"],
                              hs_token=self.config["appservice.hs_token"],
 
@@ -109,7 +128,10 @@ class Bridge(Program):
                              })
 
     def prepare_db(self) -> None:
-        self.db = sql.create_engine(self.config["appservice.database"])
+        if not sql:
+            raise RuntimeError("SQLAlchemy is not installed")
+        self.db = sql.create_engine(self.config["appservice.database"],
+                                    **self.config["appservice.database_opts"])
         Base.metadata.bind = self.db
         if not self.db.has_table("alembic_version"):
             self.log.critical("alembic_version table not found. "
@@ -117,7 +139,7 @@ class Bridge(Program):
             sys.exit(10)
 
     def prepare_bridge(self) -> None:
-        self.matrix = self.matrix_class(az=self.az, config=self.config, loop=self.loop, bridge=self)
+        self.matrix = self.matrix_class(bridge=self)
 
     async def start(self) -> None:
         self.log.debug("Starting appservice...")
@@ -131,3 +153,25 @@ class Bridge(Program):
     async def stop(self) -> None:
         await self.az.stop()
         await super().stop()
+        if self.matrix.e2ee:
+            await self.matrix.e2ee.stop()
+
+    @abstractmethod
+    async def get_user(self, user_id: UserID) -> 'BaseUser':
+        pass
+
+    @abstractmethod
+    async def get_portal(self, room_id: RoomID) -> 'BasePortal':
+        pass
+
+    @abstractmethod
+    async def get_puppet(self, user_id: UserID, create: bool = False) -> 'BasePuppet':
+        pass
+
+    @abstractmethod
+    async def get_double_puppet(self, user_id: UserID) -> 'BasePuppet':
+        pass
+
+    @abstractmethod
+    def is_bridge_ghost(self, user_id: UserID) -> bool:
+        pass
