@@ -15,7 +15,7 @@ import json
 from aiohttp import ClientConnectionError
 
 from mautrix.types import (UserID, FilterID, Filter, RoomEventFilter, RoomFilter, EventFilter,
-                           EventType, SyncToken, RoomID, Event, PresenceState)
+                           EventType, SyncToken, RoomID, Event, PresenceState, StateFilter)
 from mautrix.appservice import AppService, IntentAPI
 from mautrix.errors import IntentError, MatrixError, MatrixRequestError, MatrixInvalidToken
 from mautrix.api import Path
@@ -85,7 +85,7 @@ class CustomPuppetMixin(ABC):
     _sync_task: Optional[asyncio.Future] = None
 
     @abstractmethod
-    def save(self) -> None:
+    async def save(self) -> None:
         """Save the information of this puppet. Called from :meth:`switch_mxid`"""
 
     @property
@@ -102,24 +102,26 @@ class CustomPuppetMixin(ABC):
         return (self.az.intent.user(self.custom_mxid, self.access_token)
                 if self.is_real_user else self.default_mxid_intent)
 
-    def can_auto_login(self, mxid: UserID) -> bool:
-        if not self.login_shared_secret:
+    @classmethod
+    def can_auto_login(cls, mxid: UserID) -> bool:
+        if not cls.login_shared_secret:
             return False
-        _, server = self.az.intent.parse_user_id(mxid)
-        if server != self.az.domain:
+        _, server = cls.az.intent.parse_user_id(mxid)
+        if server != cls.az.domain:
             return False
         return True
 
-    async def _login_with_shared_secret(self, mxid: UserID) -> Optional[str]:
-        if not self.can_auto_login(mxid):
+    @classmethod
+    async def _login_with_shared_secret(cls, mxid: UserID) -> Optional[str]:
+        if not cls.can_auto_login(mxid):
             return None
-        password = hmac.new(self.login_shared_secret, mxid.encode("utf-8"),
+        password = hmac.new(cls.login_shared_secret, mxid.encode("utf-8"),
                             hashlib.sha512).hexdigest()
-        url = self.az.intent.api.base_url + str(Path.login)
-        resp = await self.az.http_session.post(url, data=json.dumps({
+        url = cls.az.intent.api.base_url + str(Path.login)
+        resp = await cls.az.http_session.post(url, data=json.dumps({
             "type": "m.login.password",
-            "initial_device_display_name": self.login_device_name,
-            "device_id": self.login_device_name,
+            "initial_device_display_name": cls.login_device_name,
+            "device_id": cls.login_device_name,
             "identifier": {
                 "type": "m.id.user",
                 "user": mxid,
@@ -161,7 +163,7 @@ class CustomPuppetMixin(ABC):
                 await self._leave_rooms_with_default_user()
             except Exception:
                 self.log.warning("Error when leaving rooms with default user", exc_info=True)
-        self.save()
+        await self.save()
 
     async def try_start(self) -> None:
         try:
@@ -201,7 +203,7 @@ class CustomPuppetMixin(ABC):
             self._sync_task.cancel()
             self._sync_task = None
 
-    def default_puppet_should_leave_room(self, room_id: RoomID) -> bool:
+    async def default_puppet_should_leave_room(self, room_id: RoomID) -> bool:
         """
         Whether or not the default puppet user should leave the given room when this puppet is
         switched to using a custom user account.
@@ -217,29 +219,30 @@ class CustomPuppetMixin(ABC):
     async def _leave_rooms_with_default_user(self) -> None:
         for room_id in await self.default_mxid_intent.get_joined_rooms():
             try:
-                if self.default_puppet_should_leave_room(room_id):
+                if await self.default_puppet_should_leave_room(room_id):
                     await self.default_mxid_intent.leave_room(room_id)
                     await self.intent.ensure_joined(room_id)
             except (IntentError, MatrixRequestError):
                 pass
 
     def _create_sync_filter(self) -> Awaitable[FilterID]:
+        all_events = EventType.find("*")
         return self.intent.create_filter(Filter(
-            account_data=EventFilter(types=[]),
+            account_data=EventFilter(types=[all_events]),
+            presence=EventFilter(
+                types=[EventType.PRESENCE],
+                senders=[self.custom_mxid] if self.only_handle_own_synced_events else None,
+            ),
             room=RoomFilter(
                 include_leave=False,
-                state=RoomEventFilter(types=[]),
-                timeline=RoomEventFilter(types=[]),
-                account_data=RoomEventFilter(types=[]),
+                state=StateFilter(not_types=[all_events]),
+                timeline=RoomEventFilter(not_types=[all_events]),
+                account_data=RoomEventFilter(not_types=[all_events]),
                 ephemeral=RoomEventFilter(types=[
                     EventType.TYPING,
                     EventType.RECEIPT,
                 ]),
             ),
-            presence=EventFilter(
-                types=[EventType.PRESENCE],
-                senders=[self.custom_mxid] if self.only_handle_own_synced_events else None,
-            )
         ))
 
     def _filter_events(self, room_id: RoomID, events: List[Dict]) -> Iterator[Event]:
@@ -304,7 +307,7 @@ class CustomPuppetMixin(ABC):
                 errors = 0
                 if cur_batch is not None:
                     self._handle_sync(sync_resp)
-            except (MatrixError, ClientConnectionError) as e:
+            except (MatrixError, ClientConnectionError, asyncio.TimeoutError) as e:
                 errors += 1
                 wait = min(errors, 11) ** 2
                 self.log.warning(f"Syncer for {custom_mxid} errored: {e}. "
