@@ -5,15 +5,15 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from typing import Awaitable, Dict, Optional, List
 
-from ...api import Method, JSON, Path
-from ...errors import MatrixResponseError
-from .types import (UserID, RoomID, EventID, FilterID, SyncToken, PaginationDirection, StateEvent,
-                    EventType, StateEventContent, MessageEventContent, Member, Event, ContentURI,
-                    PaginatedMessages, SerializerError, MessageType, RelatesTo, Format, ImageInfo,
-                    BaseFileInfo, TextMessageEventContent, MediaMessageEventContent, PresenceState,
-                    EventContent, ReactionEventContent, RelationType)
-from .types.event.state import state_event_content_map
-from .types.util import Obj, Serializable
+from mautrix.api import Method, Path
+from mautrix.errors import MatrixResponseError
+from mautrix.types import (JSON, UserID, RoomID, EventID, FilterID, SyncToken, PaginationDirection,
+                           StateEvent, EventType, StateEventContent, MessageEventContent, Member,
+                           Event, ContentURI, PaginatedMessages, SerializerError, MessageType,
+                           RelatesTo, Format, ImageInfo, BaseFileInfo, TextMessageEventContent,
+                           MediaMessageEventContent, PresenceState, EventContent, Membership,
+                           ReactionEventContent, RelationType, Obj, Serializable)
+from mautrix.types.event.state import state_event_content_map
 from .base import BaseClientAPI
 
 
@@ -129,20 +129,39 @@ class EventMethods(BaseClientAPI):
         except SerializerError as e:
             raise MatrixResponseError("Invalid state events in response") from e
 
-    async def get_members(self, room_id: RoomID) -> List[StateEvent]:
+    async def get_members(self, room_id: RoomID, at: Optional[SyncToken] = None,
+                          membership: Optional[Membership] = None,
+                          not_membership: Optional[Membership] = None) -> List[StateEvent]:
         """
         Get the list of members for a room. See also: `/members API reference`_
 
         Args:
             room_id: The ID of the room to get the member events for.
+            at: The point in time (pagination token) to return members for in the room. This token
+                can be obtained from a ``prev_batch`` token returned for each room by the sync API.
+                Defaults to the current state of the room, as determined by the server.
+            membership: The kind of membership to filter for. Defaults to no filtering if
+                unspecified. When specified alongside ``not_membership``, the two parameters create
+                an 'or' condition: either the ``membership`` is the same as membership or is not the
+                same as ``not_membership``.
+            not_membership: The kind of membership to exclude from the results. Defaults to no
+                filtering if unspecified.
 
         Returns:
             A list of most recent member events for each user.
 
         .. _/members API reference:
-            https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-rooms-roomid-members
+            https://matrix.org/docs/spec/client_server/r0.6.1#get-matrix-client-r0-rooms-roomid-members
         """
-        content = await self.api.request(Method.GET, Path.rooms[room_id].members)
+        query = {}
+        if at:
+            query["at"] = at
+        if membership:
+            query["membership"] = membership.value
+        if not_membership:
+            query["not_membership"] = not_membership.value
+        content = await self.api.request(Method.GET, Path.rooms[room_id].members,
+                                         query_params=query)
         try:
             return [StateEvent.deserialize(event) for event in content["chunk"]]
         except KeyError:
@@ -171,8 +190,10 @@ class EventMethods(BaseClientAPI):
         """
         content = await self.api.request(Method.GET, Path.rooms[room_id].joined_members)
         try:
-            return {user_id: Member.deserialize(event)
-                    for user_id, event in content["joined"].items()}
+            return {user_id: Member(membership=Membership.JOIN,
+                                    displayname=member.get("display_name", ""),
+                                    avatar_url=member.get("avatar_url", ""))
+                    for user_id, member in content["joined"].items()}
         except KeyError:
             raise MatrixResponseError("`joined` not in response.")
         except SerializerError as e:
@@ -313,11 +334,10 @@ class EventMethods(BaseClientAPI):
         """
         return self.send_message_event(room_id, EventType.ROOM_MESSAGE, content, **kwargs)
 
-    def react(self, room_id: RoomID, event_id: EventID, key: str) -> Awaitable[EventID]:
-        # TODO make this use the send_relation API instead.
+    def react(self, room_id: RoomID, event_id: EventID, key: str, **kwargs) -> Awaitable[EventID]:
         content = ReactionEventContent(relates_to=RelatesTo(rel_type=RelationType.ANNOTATION,
                                                             event_id=event_id, key=key))
-        return self.send_message_event(room_id, EventType.REACTION, content)
+        return self.send_message_event(room_id, EventType.REACTION, content, **kwargs)
 
     def send_text(self, room_id: RoomID, text: str, html: Optional[str] = None,
                   msgtype: MessageType = MessageType.TEXT, relates_to: Optional[RelatesTo] = None,
