@@ -9,4 +9,6 @@ from .request import (MatrixRequestError, MatrixUnknownRequestError, MatrixStand
                       MCaptchaInvalid, MMissingParam, MInvalidParam, MTooLarge, MExclusive,
                       MUserDeactivated)
 from .crypto import (CryptoError, EncryptionError, SessionShareError, DecryptionError,
-                     MatchingSessionDecryptionError, DeviceValidationError)
+                     MatchingSessionDecryptionError, DeviceValidationError, SessionNotFound,
+                     DuplicateMessageIndex, VerificationError, DecryptedPayloadError,
+                     MismatchingRoomError)
