@@ -9,39 +9,34 @@ import asyncio
 
 
 class SimpleLock:
-    _loop: asyncio.AbstractEventLoop
-    _future: Optional[asyncio.Future]
+    _event: asyncio.Event
     log: Optional[logging.Logger]
     message: Optional[str]
 
-    def __init__(self, message: Optional[str] = None, log: Optional[logging.Logger] = None,
-                 loop: Optional[asyncio.AbstractEventLoop] = None) -> None:
-        self._future = None
-        self._loop = loop or asyncio.get_event_loop()
+    def __init__(self, message: Optional[str] = None, log: Optional[logging.Logger] = None) -> None:
+        self._event = asyncio.Event()
+        self._event.set()
         self.log = log
         self.message = message
 
     def __enter__(self) -> None:
-        if self._future is None or self._future.done():
-            self._future = self._loop.create_future()
+        self._event.clear()
 
     async def __aenter__(self) -> None:
         self.__enter__()
 
     def __exit__(self, exc_type, exc_val, exc_tb) -> None:
-        if self._future is not None:
-            self._future.set_result(None)
-            self._future = None
+        self._event.set()
 
     def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
         self.__exit__(exc_type, exc_val, exc_tb)
 
     @property
     def locked(self) -> bool:
-        return self._future is not None and not self._future.done()
+        return not self._event.is_set()
 
     async def wait(self, task: Optional[str] = None) -> None:
-        if self._future is not None:
+        if not self._event.is_set():
             if self.log and self.message:
                 self.log.debug(self.message, task)
-            await self._future
+            await self._event.wait()
